/// <reference types="node" />
interface Watermarks {
    lines: [number, number];
    functions: [number, number];
    branches: [number, number];
    statements: [number, number];
}
export interface FileConfig {
    reporters?: string[];
    exclude?: string[];
    include?: string[];
    coverageDir?: string;
    waterMarks?: Watermarks;
}
export interface CliConfig {
    reporters?: string[];
    exclude?: string[];
    include?: string[];
    coverageDir?: string;
    command: string[];
}
export interface ResolvedConfig {
    reporters: string[];
    exclude: string[];
    include: string[];
    coverageDir: string;
    waterMarks: Watermarks;
    command: string[];
}
export interface MessageAction {
    action: "message";
    message: string;
    error?: Error;
}
export interface RunAction {
    action: "run";
    config: ResolvedConfig;
}
export declare type CliAction = MessageAction | RunAction;
export declare type ParseArgsResult = MessageAction | {
    action: "run";
    config: CliConfig;
};
/**
 * Executes the c88 CLI
 *
 * @param args CLI arguments
 * @param cwd Current working directory
 * @param proc Current process
 */
export declare function execCli(args: string[], cwd: string, proc: NodeJS.Process): Promise<number>;
export declare function getAction(args: string[], cwd: string): Promise<CliAction>;
export declare function parseArgs(args: string[]): ParseArgsResult;
export {};
